@@ -14,7 +14,7 @@ at the top-level directory.
  * \brief Driver program for PDGSSVX example
  *
  * <pre>
- * -- Distributed SuperLU routine (version 7.0) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * March 15, 2003
  * April 5, 2015
@@ -62,7 +62,7 @@ int main(int argc, char *argv[])
     int    iam, info, ldb, ldx, nrhs;
     char     **cpp, c, *postfix;
     int ii, omp_mpi_level;
-    FILE *fp, *fopen();
+    FILE *fp;
     int cpp_defs();
 
     nprow = 1;  /* Default process rows.      */
@@ -206,7 +206,6 @@ int main(int argc, char *argv[])
     }
     
     PStatPrint(&options, &stat, &grid);        /* Print the statistics. */
-    PStatFree(&stat);
 
     /* ------------------------------------------------------------
        2. NOW SOLVE ANOTHER SYSTEM WITH THE SAME A BUT DIFFERENT
@@ -214,7 +213,7 @@ int main(int argc, char *argv[])
        LUSTRUCT OBTAINED FROM A PREVIOUS FATORIZATION.
        ------------------------------------------------------------*/
     options.Fact = FACTORED; /* Indicate the factored form of A is supplied. */
-    PStatInit(&stat); /* Initialize the statistics variables. */
+    PStatClear(&stat); /* clear the statistics variables. */
 
     nrhs = 1;
     pdgssvx(&options, &A, &ScalePermstruct, b1, ldb, nrhs, &grid,
@@ -232,7 +231,6 @@ int main(int argc, char *argv[])
     }
     
     PStatPrint(&options, &stat, &grid);        /* Print the statistics. */
-    PStatFree(&stat);
 
     /* ------------------------------------------------------------
        3. SOLVE ANOTHER SYSTEM WITH THE SAME A BUT DIFFERENT
@@ -240,7 +238,7 @@ int main(int argc, char *argv[])
        FACTORS IN LUSTRUCT OBTAINED FROM A PREVIOUS FATORIZATION.
        ------------------------------------------------------------*/
     options.Fact = FACTORED; /* Indicate the factored form of A is supplied. */
-    PStatInit(&stat); /* Initialize the statistics variables. */
+    PStatClear(&stat); /* clear the statistics variables. */
 
     nrhs = 3;
     
@@ -269,11 +267,11 @@ int main(int argc, char *argv[])
     }
     
     PStatPrint(&options, &stat, &grid);        /* Print the statistics. */
-    PStatFree(&stat);
 
     /* ------------------------------------------------------------
        DEALLOCATE STORAGE.
        ------------------------------------------------------------*/
+    PStatFree(&stat);
     Destroy_CompRowLoc_Matrix_dist(&A);
     dScalePermstructFree(&ScalePermstruct);   
     dDestroy_LU(n, &grid, &LUstruct);
