@@ -14,7 +14,7 @@ at the top-level directory.
  * \brief This example illustrates how to divide up the processes into subgroups
  *
  * <pre>
- * -- Distributed SuperLU routine (version 4.1) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * September 1, 1999
  * April 5, 2015
@@ -65,7 +65,7 @@ int main(int argc, char *argv[])
     int      nrhs = 1;   /* Number of right-hand side. */
     char     trans[1];
     char     **cpp, c;
-    FILE *fp, *fopen();
+    FILE *fp;
 
     /* ------------------------------------------------------------
        INITIALIZE MPI ENVIRONMENT. 
@@ -177,8 +177,17 @@ int main(int argc, char *argv[])
 	*trans = 'N';
 	ldx = n;
 	ldb = m;
-	dGenXtrue_dist(n, nrhs, xtrue, ldx);
-	dFillRHS_dist(trans, nrhs, xtrue, ldx, &A, b, ldb);
+
+  	if ( iam==0 ) {
+	    dGenXtrue_dist(n, nrhs, xtrue, ldx);
+	    dFillRHS_dist(trans, nrhs, xtrue, ldx, &A, b, ldb);
+	    
+            MPI_Bcast( xtrue, n*nrhs, MPI_DOUBLE, 0, grid1.comm );
+            MPI_Bcast( b, m*nrhs, MPI_DOUBLE, 0, grid1.comm );
+	} else {
+            MPI_Bcast( xtrue, n*nrhs, MPI_DOUBLE, 0, grid1.comm );
+            MPI_Bcast( b, m*nrhs, MPI_DOUBLE, 0, grid1.comm );
+	}
 
 	if ( !(berr = doubleMalloc_dist(nrhs)) )
 	    ABORT("Malloc fails for berr[].");
@@ -282,8 +291,16 @@ int main(int argc, char *argv[])
 	*trans = 'N';
 	ldx = n;
 	ldb = m;
-	dGenXtrue_dist(n, nrhs, xtrue, ldx);
-	dFillRHS_dist(trans, nrhs, xtrue, ldx, &A, b, ldb);
+
+        if ( iam==0 ) {
+	    dGenXtrue_dist(n, nrhs, xtrue, ldx);
+	    dFillRHS_dist(trans, nrhs, xtrue, ldx, &A, b, ldb);
+            MPI_Bcast( xtrue, n*nrhs, MPI_DOUBLE, 0, grid2.comm );
+            MPI_Bcast( b, m*nrhs, MPI_DOUBLE, 0, grid2.comm );
+	} else {
+            MPI_Bcast( xtrue, n*nrhs, MPI_DOUBLE, 0, grid2.comm );
+            MPI_Bcast( b, m*nrhs, MPI_DOUBLE, 0, grid2.comm );
+	}
 
 	if ( !(berr = doubleMalloc_dist(nrhs)) )
 	    ABORT("Malloc fails for berr[].");
