@@ -1,19 +1,19 @@
 /*! \file
 Copyright (c) 2003, The Regents of the University of California, through
-Lawrence Berkeley National Laboratory (subject to receipt of any required 
-approvals from U.S. Dept. of Energy) 
+Lawrence Berkeley National Laboratory (subject to receipt of any required
+approvals from U.S. Dept. of Energy)
 
-All rights reserved. 
+All rights reserved.
 
 The source code is distributed under BSD license, see the file License.txt
 at the top-level directory.
 */
 
-/*! @file 
+/*! @file
  * \brief Driver program for PZGSSVX example
  *
  * <pre>
- * -- Distributed SuperLU routine (version 8.1.0) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * November 1, 2007
  * December 6, 2018
@@ -34,7 +34,7 @@ at the top-level directory.
  *
  * This example illustrates how to use PZGSSVX with the full
  * (default) options to solve a linear system.
- * 
+ *
  * Five basic steps are required:
  *   1. Initialize the MPI environment and the SuperLU process grid
  *   2. Set up the input matrix and the right-hand side
@@ -62,7 +62,7 @@ int main(int argc, char *argv[])
     int      nprow, npcol, lookahead, colperm, rowperm, ir, symbfact, batch;
     int      iam, info, ldb, ldx, nrhs;
     char     **cpp, c, *postfix;;
-    FILE *fp, *fopen();
+    FILE *fp;
     int cpp_defs();
     int ii, omp_mpi_level;
     int ldumap, myrank, p; /* The following variables are used for batch solves */
@@ -86,21 +86,44 @@ int main(int argc, char *argv[])
     batch = 0;
 
     /* ------------------------------------------------------------
-       INITIALIZE MPI ENVIRONMENT. 
+       INITIALIZE MPI ENVIRONMENT.
        ------------------------------------------------------------*/
     //MPI_Init( &argc, &argv );
-    MPI_Init_thread( &argc, &argv, MPI_THREAD_MULTIPLE, &omp_mpi_level); 
-	
+    MPI_Init_thread( &argc, &argv, MPI_THREAD_MULTIPLE, &omp_mpi_level);
+
 
 #if ( VAMPIR>=1 )
-    VT_traceoff(); 
+    VT_traceoff();
 #endif
 
 #if ( VTUNE>=1 )
 	__itt_pause();
 #endif
-	
-    /* Parse command line argv[]. */
+
+    /* Set the default input options:
+        options.Fact              = DOFACT;
+        options.Equil             = YES;
+        options.ParSymbFact       = NO;
+        options.ColPerm           = METIS_AT_PLUS_A;
+        options.RowPerm           = LargeDiag_MC64;
+        options.ReplaceTinyPivot  = NO;
+        options.IterRefine        = SLU_DOUBLE;
+        options.Trans             = NOTRANS;
+        options.SolveInitialized  = NO;
+        options.RefineInitialized = NO;
+        options.PrintStat         = YES;
+	options.DiagInv           = NO;
+     */
+    set_default_options_dist(&options);
+#if 0
+    options.RowPerm = LargeDiag_HWPM;
+    options.IterRefine = NOREFINE;
+    options.ColPerm = NATURAL;
+    options.Equil = NO;
+    options.ReplaceTinyPivot = YES;
+#endif
+
+    /* Parse command line argv[], may modify default options */
     for (cpp = argv+1; *cpp; ++cpp) {
 	if ( **cpp == '-' ) {
 	    c = *(*cpp+1);
@@ -108,8 +131,14 @@ int main(int argc, char *argv[])
 	    switch (c) {
 	      case 'h':
 		  printf("Options:\n");
-		  printf("\t-r <int>: process rows    (default %4d)\n", nprow);
-		  printf("\t-c <int>: process columns (default %4d)\n", npcol);
+		  printf("\t-r <int>: process rows       (default %4d)\n", nprow);
+		  printf("\t-c <int>: process columns    (default %4d)\n", npcol);
+		  printf("\t-p <int>: row permutation    (default %4d)\n", options.RowPerm);
+		  printf("\t-q <int>: col permutation    (default %4d)\n", options.ColPerm);
+		  printf("\t-s <int>: parallel symbolic? (default %4d)\n", options.ParSymbFact);
+		  printf("\t-l <int>: lookahead level    (default %4d)\n", options.num_lookaheads);
+		  printf("\t-i <int>: iter. refinement   (default %4d)\n", options.IterRefine);
+		  printf("\t-b <int>: use batch mode?    (default %4d)\n", batch);
 		  exit(0);
 		  break;
 	      case 'r': nprow = atoi(*cpp);
@@ -137,24 +166,35 @@ int main(int argc, char *argv[])
 	}
     }
 
-    if ( batch ) { /* in the batch mode: create multiple SuperLU grids,
-		      each grid solving one linear system. */
+    /* Command line input to modify default options */
+    if (rowperm != -1) options.RowPerm = rowperm;
+    if (colperm != -1) options.ColPerm = colperm;
+    if (lookahead != -1) options.num_lookaheads = lookahead;
+    if (ir != -1) options.IterRefine = ir;
+    if (symbfact != -1) options.ParSymbFact = symbfact;
+
+    int superlu_acc_offload = sp_ienv_dist(10, &options); //get_acc_offload();
+    
+    /* In the batch mode: create multiple SuperLU grids,
+        each grid solving one linear system. */
+    if ( batch ) {
 	/* ------------------------------------------------------------
-	   INITIALIZE MULTIPLE SUPERLU PROCESS GRIDS. 
+	   INITIALIZE MULTIPLE SUPERLU PROCESS GRIDS.
 	   ------------------------------------------------------------*/
         MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
         usermap = SUPERLU_MALLOC(nprow*npcol * sizeof(int));
         ldumap = nprow;
-        int color = myrank/(nprow*npcol); /* Assuming each grid uses the same number of nprow and npcol */
+
+        /* Assuming each grid uses the same number of nprow and npcol */
+	int color = myrank/(nprow*npcol);
 	MPI_Comm_split(MPI_COMM_WORLD, color, myrank, &SubComm);
-        p = 0;    
+        p = 0;
         for (int i = 0; i < nprow; ++i)
     	    for (int j = 0; j < npcol; ++j) usermap[i+j*ldumap] = p++;
         superlu_gridmap(SubComm, nprow, npcol, usermap, ldumap, &grid);
         SUPERLU_FREE(usermap);
 
 #ifdef GPU_ACC
-        int superlu_acc_offload = get_acc_offload();
         if (superlu_acc_offload) {
             /* Binding each MPI to a GPU device */
             char *ttemp;
@@ -167,12 +207,12 @@ int main(int argc, char *argv[])
 	        gpuSetDevice(rank % devs); // Set device to be used for GPU executions
             }
 
-            // This is to initialize GPU, which can be costly. 
-            double t1 = SuperLU_timer_();                       
+            // This is to initialize GPU, which can be costly.
+            double t1 = SuperLU_timer_();
             gpuFree(0);
-            double t2 = SuperLU_timer_();    
+            double t2 = SuperLU_timer_();
             if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
-            gpublasHandle_t hb;           
+            gpublasHandle_t hb;
             gpublasCreate(&hb);
             if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
             gpublasDestroy(hb);
@@ -181,28 +221,27 @@ int main(int argc, char *argv[])
         // printf("grid.iam %5d, myrank %5d\n",grid.iam,myrank);
         // fflush(stdout);
 
-    } else {
+    } else { /* not batch mode */
         /* ------------------------------------------------------------
            INITIALIZE THE SUPERLU PROCESS GRID.
            ------------------------------------------------------------ */
         superlu_gridinit(MPI_COMM_WORLD, nprow, npcol, &grid);
-	
+
 #ifdef GPU_ACC
-        int superlu_acc_offload = get_acc_offload();
         if (superlu_acc_offload) {
             MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
-            double t1 = SuperLU_timer_();                       
+            double t1 = SuperLU_timer_();
             gpuFree(0);
-            double t2 = SuperLU_timer_();    
+            double t2 = SuperLU_timer_();
             if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
-            gpublasHandle_t hb;           
+            gpublasHandle_t hb;
             gpublasCreate(&hb);
             if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
             gpublasDestroy(hb);
 	}
 #endif
     }
-    
+
     if(grid.iam==0){
 	MPI_Query_thread(&omp_mpi_level);
         switch (omp_mpi_level) {
@@ -224,7 +263,7 @@ int main(int argc, char *argv[])
 	        break;
         }
     }
-	
+
     /* Bail out if I do not belong in the grid. */
     iam = grid.iam;
     if ( (iam >= nprow * npcol) || (iam == -1) ) goto out;
@@ -243,6 +282,12 @@ int main(int argc, char *argv[])
 	fflush(stdout);
     }
 
+    /* print solver options */
+    if (!iam) {
+	print_options_dist(&options);
+	fflush(stdout);
+    }
+
 #if ( VAMPIR>=1 )
     VT_traceoff();
 #endif
@@ -257,9 +302,9 @@ int main(int argc, char *argv[])
 	}
     }
     // printf("%s\n", postfix);
-	
+
     /* ------------------------------------------------------------
-       GET THE MATRIX FROM FILE AND SETUP THE RIGHT HAND SIDE. 
+       GET THE MATRIX FROM FILE AND SETUP THE RIGHT HAND SIDE.
        ------------------------------------------------------------*/
     zcreate_matrix_postfix(&A, nrhs, &b, &ldb, &xtrue, &ldx, fp, postfix, &grid);
 
@@ -270,39 +315,6 @@ int main(int argc, char *argv[])
        NOW WE SOLVE THE LINEAR SYSTEM.
        ------------------------------------------------------------*/
 
-    /* Set the default input options:
-        options.Fact              = DOFACT;
-        options.Equil             = YES;
-        options.ParSymbFact       = NO;
-        options.ColPerm           = METIS_AT_PLUS_A;
-        options.RowPerm           = LargeDiag_MC64;
-        options.ReplaceTinyPivot  = NO;
-        options.IterRefine        = SLU_DOUBLE;
-        options.Trans             = NOTRANS;
-        options.SolveInitialized  = NO;
-        options.RefineInitialized = NO;
-        options.PrintStat         = YES;
-	options.DiagInv           = NO;
-     */
-    set_default_options_dist(&options);
-#if 0
-    options.RowPerm = LargeDiag_HWPM;
-    options.IterRefine = NOREFINE;
-    options.ColPerm = NATURAL;
-    options.Equil = NO; 
-    options.ReplaceTinyPivot = YES;
-#endif
-
-    if (rowperm != -1) options.RowPerm = rowperm;
-    if (colperm != -1) options.ColPerm = colperm;
-    if (lookahead != -1) options.num_lookaheads = lookahead;
-    if (ir != -1) options.IterRefine = ir;
-    if (symbfact != -1) options.ParSymbFact = symbfact;
-
-    if (!iam) {
-	print_options_dist(&options);
-	fflush(stdout);
-    }
 
     m = A.nrow;
     n = A.ncol;
@@ -350,10 +362,10 @@ int main(int argc, char *argv[])
        ------------------------------------------------------------*/
 out:
     if ( batch ) {
-        result_min[0] = stat.utime[FACT];   
-        result_min[1] = stat.utime[SOLVE];  
-        result_max[0] = stat.utime[FACT];   
-        result_max[1] = stat.utime[SOLVE];    
+        result_min[0] = stat.utime[FACT];
+        result_min[1] = stat.utime[SOLVE];
+        result_max[0] = stat.utime[FACT];
+        result_max[1] = stat.utime[SOLVE];
         MPI_Allreduce(MPI_IN_PLACE, result_min, 2, MPI_FLOAT,MPI_MIN, MPI_COMM_WORLD);
         MPI_Allreduce(MPI_IN_PLACE, result_max, 2, MPI_FLOAT,MPI_MAX, MPI_COMM_WORLD);
         if (!myrank) {
@@ -364,10 +376,10 @@ out:
             fflush(stdout);
         }
     }
-    
+
     superlu_gridexit(&grid);
     if ( iam != -1 ) PStatFree(&stat);
-    
+
     /* ------------------------------------------------------------
        TERMINATES THE MPI EXECUTION ENVIRONMENT.
        ------------------------------------------------------------*/
