@@ -13,7 +13,7 @@ at the top-level directory.
  * \brief Driver program for PZGSSVX example
  *
  * <pre>
- * -- Distributed SuperLU routine (version 6.1) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * March 15, 2003
  * April 5, 2015
@@ -63,7 +63,7 @@ int main(int argc, char *argv[])
     int      iam, info, ldb, ldx, nrhs;
     char     **cpp, c, *postfix;
     int ii, omp_mpi_level;
-    FILE *fp, *fopen();
+    FILE *fp;
     int cpp_defs();
 
     /* prototypes */
@@ -200,7 +200,6 @@ int main(int argc, char *argv[])
     }
     
     PStatPrint(&options, &stat, &grid);        /* Print the statistics. */
-    PStatFree(&stat);
     Destroy_CompRowLoc_Matrix_dist(&A); /* Deallocate storage of matrix A.  */ 
     zDestroy_LU(n, &grid, &LUstruct); /* Deallocate storage associated with 
 					the L and U matrices.  */
@@ -227,7 +226,7 @@ int main(int argc, char *argv[])
     zcreate_matrix_perturbed_postfix(&A, nrhs, &b1, &ldb,
                                   &xtrue1, &ldx, fp, postfix, &grid);
 			     
-    PStatInit(&stat); /* Initialize the statistics variables. */
+    PStatClear(&stat); /* clear the statistics variables. */
 
     /* Solve the linear system. */
     pzgssvx(&options, &A, &ScalePermstruct, b1, ldb, nrhs, &grid,
