@@ -1,37 +1,34 @@
 /*! \file
 Copyright (c) 2003, The Regents of the University of California, through
-Lawrence Berkeley National Laboratory (subject to receipt of any required 
-approvals from U.S. Dept. of Energy) 
+Lawrence Berkeley National Laboratory (subject to receipt of any required
+approvals from U.S. Dept. of Energy)
 
-All rights reserved. 
+All rights reserved.
 
 The source code is distributed under BSD license, see the file License.txt
 at the top-level directory.
 */
 
-/*! @file
- * \brief Driver program for PZGSSVX3D example
- *
+/*
  * <pre>
- * -- Distributed SuperLU routine (version 8.1.0) --
+ * -- Distributed SuperLU routine (version 9.0) --
  * Lawrence Berkeley National Lab, Georgia Institute of Technology,
- * Oak Ridge National Lab 
+ * Oak Ridge National Lab
  * May 12, 2021
  * August 27, 2022  Add batch option
+ * January 15, 2024 Complete the batch interface
  *
  */
-#include "superlu_zdefs.h"  
+#include "superlu_zdefs.h"
 
-/*! \brief
+/*! \brief The driver program PZDRIVE3D.
  *
  * <pre>
  * Purpose
  * =======
  *
- * The driver program PZDRIVE3D.
- *
- * This example illustrates how to use PZGSSVX3D with the full
- * (default) options to solve a linear system.
+ * This example illustrates how to use PZGSSVX3D or PZGSSVX3D_CSC_BATCH
+ * with the full (default) options to solve a linear system.
  *
  * Five basic steps are required:
  *   1. Initialize the MPI environment and the SuperLU process grid
@@ -48,7 +45,7 @@ at the top-level directory.
  *
  * </pre>
  */
- 
+
 static void matCheck(int n, int m, doublecomplex* A, int LDA,
        doublecomplex* B, int LDB)
 {
@@ -101,12 +98,11 @@ static void checkNRFMT(NRformat_loc*A, NRformat_loc*B)
 
 }
 
-int
-main (int argc, char *argv[])
+int main (int argc, char *argv[])
 {
     superlu_dist_options_t options;
     SuperLUStat_t stat;
-    SuperMatrix A;  // Now, A is on all 3D processes  
+    SuperMatrix A;  // Now, A is on all 3D processes
     zScalePermstruct_t ScalePermstruct;
     zLUstruct_t LUstruct;
     zSOLVEstruct_t SOLVEstruct;
@@ -115,12 +111,12 @@ main (int argc, char *argv[])
     doublecomplex *b, *xtrue;
     int_t m, n;
     int nprow, npcol, npdep;
-    int lookahead, colperm, rowperm, ir, batch;
+    int equil, colperm, rowperm, ir, lookahead;
     int iam, info, ldb, ldx, nrhs;
     char **cpp, c, *suffix;
-    FILE *fp, *fopen ();
+    FILE *fp;
     extern int cpp_defs ();
-    int ii, omp_mpi_level;
+    int ii, omp_mpi_level, batchCount = 0;
     int*    usermap;     /* The following variables are used for batch solves */
     float result_min[2];
     result_min[0]=1e10;
@@ -135,12 +131,12 @@ main (int argc, char *argv[])
     npcol = 1;            /* Default process columns.   */
     npdep = 1;            /* replication factor must be power of two */
     nrhs = 1;             /* Number of right-hand side. */
-    lookahead = -1;
+    equil = -1;
     colperm = -1;
     rowperm = -1;
     ir = -1;
-    batch = 0;
-    
+    lookahead = -1;
+
     /* ------------------------------------------------------------
        INITIALIZE MPI ENVIRONMENT.
        ------------------------------------------------------------ */
@@ -183,7 +179,9 @@ main (int argc, char *argv[])
             case 'd':
                 npdep = atoi (*cpp);
                 break;
-            case 'l': lookahead = atoi(*cpp);
+            case 'b': batchCount = atoi(*cpp);
+                      break;
+            case 'e': equil = atoi(*cpp);
                       break;
             case 'p': rowperm = atoi(*cpp);
                       break;
@@ -191,7 +189,9 @@ main (int argc, char *argv[])
                       break;
             case 'i': ir = atoi(*cpp);
                       break;
-            case 'b': batch = atoi(*cpp);
+            case 's': nrhs = atoi(*cpp);
+                      break;                      
+            case 'l': lookahead = atoi(*cpp);
                       break;
             }
         }
@@ -205,65 +205,87 @@ main (int argc, char *argv[])
         }
     }
 
-    if ( batch ) { /* in the batch mode: create multiple SuperLU grids,
-		      each grid solving one linear system. */
-	/* ------------------------------------------------------------
-	   INITIALIZE MULTIPLE SUPERLU PROCESS GRIDS. 
-	   ------------------------------------------------------------*/
-	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
-	usermap = SUPERLU_MALLOC(nprow*npcol*npdep * sizeof(int));
-	int color = myrank/(nprow*npcol*npdep); /* Assuming each grid uses the same number of nprow, npcol and npdep */
-	MPI_Comm_split(MPI_COMM_WORLD, color, myrank, &SubComm);
-	p = 0;
-	for (int k = 0; k < npdep; ++k) 
-	    for (int i = 0; i < nprow; ++i)
-		for (int j = 0; j < npcol; ++j) usermap[i + j*nprow + k*nprow*npcol] = p++;
-	superlu_gridmap3d(SubComm, nprow, npcol, npdep, usermap, &grid);
-	SUPERLU_FREE(usermap);
-
-#ifdef GPU_ACC
-	/* Binding each MPI to a GPU device */
-	char *ttemp;
-	ttemp = getenv ("SUPERLU_BIND_MPI_GPU");
-
-	if (ttemp) {
-	    int devs, rank;
-	    MPI_Comm_rank(MPI_COMM_WORLD, &rank); // MPI_COMM_WORLD needs to be used here instead of SubComm
-	    gpuGetDeviceCount(&devs);  // Returns the number of compute-capable devices
-	    gpuSetDevice(rank % devs); // Set device to be used for GPU executions
-	}
-        // This is to initialize GPU, which can be costly. 
-        double t1 = SuperLU_timer_();                       
-        gpuFree(0);
-        double t2 = SuperLU_timer_();    
-        if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
-        gpublasHandle_t hb;           
-        gpublasCreate(&hb);
-        if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
-        gpublasDestroy(hb);
-
+    /* Set the default input options:
+       options.Fact              = DOFACT;
+       options.Equil             = YES;
+       options.ParSymbFact       = NO;
+       options.ColPerm           = METIS_AT_PLUS_A;
+       options.RowPerm           = LargeDiag_MC64;
+       options.ReplaceTinyPivot  = NO;
+       options.IterRefine        = SLU_DOUBLE;
+       options.Trans             = NOTRANS;
+       options.SolveInitialized  = NO;
+       options.RefineInitialized = NO;
+       options.PrintStat         = YES;
+       options->num_lookaheads    = 10;
+       options->lookahead_etree   = NO;
+       options->SymPattern        = NO;
+       options.DiagInv           = NO;
+     */
+    set_default_options_dist (&options);
+    options.IterRefine = NOREFINE;
+#if 0
+    options.ILU_level = 0;
+    options.ParSymbFact       = YES;
+    options.ColPerm           = PARMETIS;
+    options.DiagInv           = YES; // only if SLU_HAVE_LAPACK
+    options.ReplaceTinyPivot = YES;
+    options.RowPerm = NOROWPERM;
+    options.ColPerm = NATURAL;
+    options.ReplaceTinyPivot = YES;
 #endif
 
-	// printf("grid.iam %5d, myrank %5d\n",grid.iam,myrank);
-	// fflush(stdout);
+    if ( batchCount > 0 )
+        options.batchCount = batchCount;
+
+    if (equil != -1) options.Equil = equil;
+    if (rowperm != -1) options.RowPerm = rowperm;
+    if (colperm != -1) options.ColPerm = colperm;
+    if (ir != -1) options.IterRefine = ir;
+    if (lookahead != -1) options.num_lookaheads = lookahead;
+
+    //////* this test SolveOnly*/
+    // options.SolveOnly = YES;
 	
-    } else {
-        /* ------------------------------------------------------------
-           INITIALIZE THE SUPERLU PROCESS GRID.
-           ------------------------------------------------------------ */
-        superlu_gridinit3d (MPI_COMM_WORLD, nprow, npcol, npdep, &grid);
+    //////* this test everything in SolveOnly except ILU_level = 0*/
+    // options.Equil = NO;
+	// options.RowPerm = NOROWPERM;
+	// options.ColPerm = NATURAL;
+
+    /* ------------------------------------------------------------
+       INITIALIZE THE SUPERLU PROCESS GRID.
+       ------------------------------------------------------------ */
+    superlu_gridinit3d (MPI_COMM_WORLD, nprow, npcol, npdep, &grid);
+    iam = grid.iam;
+
+    if (!iam) {
+	print_sp_ienv_dist(&options);
+	print_options_dist(&options);
+	fflush(stdout);
+    }
+    
+#if ( DEBUGlevel>=1 )
+    CHECK_MALLOC (iam, "Enter main()");
+#endif
+
+
 #ifdef GPU_ACC
+    /* ------------------------------------------------------------
+       INITIALIZE GPU ENVIRONMENT
+       ------------------------------------------------------------ */
+    int superlu_acc_offload = sp_ienv_dist(10, &options); //get_acc_offload();
+    if (superlu_acc_offload) {
         MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
-        double t1 = SuperLU_timer_();                       
+        double t1 = SuperLU_timer_();
         gpuFree(0);
-        double t2 = SuperLU_timer_();    
+        double t2 = SuperLU_timer_();
         if(!myrank)printf("first gpufree time: %7.4f\n",t2-t1);
-        gpublasHandle_t hb;           
+        gpublasHandle_t hb;
         gpublasCreate(&hb);
         if(!myrank)printf("first blas create time: %7.4f\n",SuperLU_timer_()-t2);
         gpublasDestroy(hb);
+	}
 #endif
-    }
 
     if(grid.iam==0) {
 	MPI_Query_thread(&omp_mpi_level);
@@ -289,7 +311,6 @@ main (int argc, char *argv[])
     }
 	
     /* Bail out if I do not belong in the grid. */
-    iam = grid.iam;
     if (iam == -1)     goto out;
     if (!iam) {
 	int v_major, v_minor, v_bugfix;
@@ -307,10 +328,6 @@ main (int argc, char *argv[])
 	fflush(stdout);
     }
 
-#if ( DEBUGlevel>=1 )
-    CHECK_MALLOC (iam, "Enter main()");
-#endif
-
     /* ------------------------------------------------------------
        GET THE MATRIX FROM FILE AND SETUP THE RIGHT HAND SIDE.
        ------------------------------------------------------------ */
@@ -321,17 +338,112 @@ main (int argc, char *argv[])
 	}
     }
 
+    if ( batchCount > 0 ) {
+	/* ------------------------------------------------------------
+	   SOLVE THE BATCH LINEAR SYSTEM.
+	   ------------------------------------------------------------ */
+	printf("batchCount %d\n", batchCount);
+	// dcreate_block_diag_3d(&A, batchCount, nrhs, &b, &ldb, &xtrue, &ldx, fp, suffix, &grid);
+	
+	handle_t *F;
+	doublecomplex **RHSptr;
+	int *ldRHS;
+	double **ReqPtr;
+	double **CeqPtr;
+	DiagScale_t *DiagScale;
+	int **RpivPtr;
+	int **CpivPtr;
+	doublecomplex **Xptr;
+	int *ldX;
+	doublecomplex **xtrues;
+	double **Berrs;
+	
+	handle_t *SparseMatrix_handles = SUPERLU_MALLOC( batchCount *  sizeof(handle_t) );
+	RHSptr = (doublecomplex **) SUPERLU_MALLOC( batchCount *  sizeof(doublecomplex *) );
+	ldRHS = int32Malloc_dist(batchCount);
+	xtrues = (doublecomplex **) SUPERLU_MALLOC( batchCount *  sizeof(doublecomplex *) );
+	ldX = int32Malloc_dist(batchCount);
+
+	/* This creates identical copies in the batch */
+	zcreate_batch_systems(SparseMatrix_handles, batchCount, nrhs, RHSptr, ldRHS,
+			      xtrues, ldX, fp, suffix, &grid);
+
+	SuperMatrix *A = (SuperMatrix *) SparseMatrix_handles[0];
+	NCformat *Astore = A->Store;
+	doublecomplex *a = Astore->nzval;
+	m = A->nrow;
+	n = A->ncol;
+	
+	ReqPtr = (double **) SUPERLU_MALLOC( batchCount * sizeof(double *) );
+	CeqPtr = (double **) SUPERLU_MALLOC( batchCount * sizeof(double *) );
+	RpivPtr = (int **) SUPERLU_MALLOC( batchCount * sizeof(int *) );
+	CpivPtr = (int **) SUPERLU_MALLOC( batchCount * sizeof(int *) );
+	DiagScale = (DiagScale_t *) SUPERLU_MALLOC( batchCount * sizeof(DiagScale_t) );
+	Xptr = (doublecomplex **) SUPERLU_MALLOC( batchCount * sizeof(doublecomplex*) );
+	Berrs = (double **) SUPERLU_MALLOC( batchCount * sizeof(double *) );
+	for (int d = 0; d < batchCount; ++d) {
+	    DiagScale[d] = NOEQUIL;
+	    RpivPtr[d] = int32Malloc_dist(m);
+	    CpivPtr[d] = int32Malloc_dist(n);
+	    Xptr[d] = doublecomplexMalloc_dist( n *  nrhs );
+	    Berrs[d] = doubleMalloc_dist( nrhs );
+	}
+
+	/* Initialize the statistics variables. */
+	PStatInit (&stat);
+	
+	/* Call batch solver */
+	pzgssvx3d_csc_batch(&options, batchCount,
+			    m, n, Astore->nnz, nrhs, SparseMatrix_handles,
+			    RHSptr, ldRHS, ReqPtr, CeqPtr, RpivPtr, CpivPtr,
+			    DiagScale, F, Xptr, ldX, Berrs, &grid, &stat, &info);
+
+	printf("**** Backward errors ****\n");
+	for (int d = 0; d < batchCount; ++d) {
+	    printf("\tSystem %d: Berr = %e\n", d, Berrs[d][0]);
+	    //printf("\t\tDiagScale[%d] %d\n", d, DiagScale[d]);
+	}
+	
+	/* Free matrices pointed to by the handles, and ReqPtr[], etc. */
+	for (int d = 0; d < batchCount; ++d) {
+	    if ( DiagScale[d] == ROW || DiagScale[d] == BOTH )
+		SUPERLU_FREE(ReqPtr[d]);
+	    if ( DiagScale[d] == COL || DiagScale[d] == BOTH )
+		SUPERLU_FREE(CeqPtr[d]);
+	    SUPERLU_FREE(RpivPtr[d]);
+	    SUPERLU_FREE(CpivPtr[d]);
+	    SUPERLU_FREE(Xptr[d]);
+	    SUPERLU_FREE(Berrs[d]);
+	    A = (SuperMatrix *) SparseMatrix_handles[d];
+	    //	    Destroy_CompRowLoc_Matrix_dist (A);
+	}
+	SUPERLU_FREE(SparseMatrix_handles);
+	SUPERLU_FREE(RHSptr);
+	SUPERLU_FREE(ldRHS);
+	SUPERLU_FREE(xtrues);
+	SUPERLU_FREE(ldX);
+	SUPERLU_FREE(ReqPtr);
+	SUPERLU_FREE(CeqPtr);
+	SUPERLU_FREE(RpivPtr);
+	SUPERLU_FREE(CpivPtr);
+	SUPERLU_FREE(DiagScale);
+	SUPERLU_FREE(Xptr);
+	SUPERLU_FREE(Berrs);
+
+	goto out;
+	
+    } else {
+    
 #define NRFRMT
 #ifndef NRFRMT
-    if ( grid.zscp.Iam == 0 )  // only in process layer 0
-	zcreate_matrix_postfix(&A, nrhs, &b, &ldb, &xtrue, &ldx, fp, suffix, &(grid.grid2d));
-	
+        if ( grid.zscp.Iam == 0 )  // only in process layer 0
+	    zcreate_matrix_postfix(&A, nrhs, &b, &ldb, &xtrue, &ldx, fp, suffix, &(grid.grid2d));
+
 #else
-    // *fp0 = *fp;
-    zcreate_matrix_postfix3d(&A, nrhs, &b, &ldb,
+        zcreate_matrix_postfix3d(&A, nrhs, &b, &ldb,
                              &xtrue, &ldx, fp, suffix, &(grid));
-    //printf("ldx %d, ldb %d\n", ldx, ldb);
-    
+    }
+
 #if 0  // following code is only for checking *Gather* routine
     NRformat_loc *Astore, *Astore0;
     doublecomplex* B2d;
@@ -345,26 +457,26 @@ main (int argc, char *argv[])
     if ( grid.zscp.Iam == 0 )  // only in process layer 0
     {
         zcreate_matrix_postfix(&Aref, nrhs, &bref, &ldb,
-                               &xtrueref, &ldx, fp0, 
+                               &xtrueref, &ldx, fp0,
                                suffix, &(grid.grid2d));
         Astore0 = (NRformat_loc *) Aref.Store;
 
 	/*
 	if ( (grid.grid2d).iam == 0 ) {
-	    printf(" iam %d\n", 0); 
+	    printf(" iam %d\n", 0);
 	    checkNRFMT(Astore, Astore0);
 	} else if ((grid.grid2d).iam == 1 ) {
-	    printf(" iam %d\n", 1); 
+	    printf(" iam %d\n", 1);
 	    checkNRFMT(Astore, Astore0);
-	} 
+	}
 	*/
-    
+
 	// bref, xtrueref are created on 2D
         matCheck(Astore->m_loc, nrhs, B2d, Astore->m_loc, bref, ldb);
     }
     // MPI_Finalize(); exit(0);
     #endif
-#endif
+#endif  // end if 0
 
     if (!(berr = doubleMalloc_dist (nrhs)))
         ABORT ("Malloc fails for berr[].");
@@ -373,44 +485,6 @@ main (int argc, char *argv[])
        NOW WE SOLVE THE LINEAR SYSTEM.
        ------------------------------------------------------------ */
 
-    /* Set the default input options:
-       options.Fact              = DOFACT;
-       options.Equil             = YES;
-       options.ParSymbFact       = NO;
-       options.ColPerm           = METIS_AT_PLUS_A;
-       options.RowPerm           = LargeDiag_MC64;
-       options.ReplaceTinyPivot  = NO;
-       options.IterRefine        = SLU_DOUBLE;
-       options.Trans             = NOTRANS;
-       options.SolveInitialized  = NO;
-       options.RefineInitialized = NO;
-       options.PrintStat         = YES;
-       options->num_lookaheads    = 10;
-       options->lookahead_etree   = NO;
-       options->SymPattern        = NO;
-       options.DiagInv           = NO;
-     */
-    set_default_options_dist (&options);
-    options.Algo3d = YES;
-#if 0
-    options.RowPerm = NOROWPERM;
-    options.IterRefine = NOREFINE;
-    options.ColPerm = NATURAL;
-    options.Equil = NO;
-    options.ReplaceTinyPivot = YES;
-#endif
-
-    if (rowperm != -1) options.RowPerm = rowperm;
-    if (colperm != -1) options.ColPerm = colperm;
-    if (lookahead != -1) options.num_lookaheads = lookahead;
-    if (ir != -1) options.IterRefine = ir;
-    
-    if (!iam) {
-    	//print_sp_ienv_dist(&options);
-	print_options_dist(&options);
-	fflush(stdout);
-    }
-
 #ifdef NRFRMT  // matrix is on 3D process grid
     m = A.nrow;
     n = A.ncol;
@@ -423,7 +497,7 @@ main (int argc, char *argv[])
     // broadcast m, n to all the process layers;
     MPI_Bcast( &m, 1, mpi_int_t, 0,  grid.zscp.comm);
     MPI_Bcast( &n, 1, mpi_int_t, 0,  grid.zscp.comm);
-#endif    
+#endif
 
     /* Initialize ScalePermstruct and LUstruct. */
     zScalePermstructInit (m, n, &ScalePermstruct);
@@ -452,16 +526,11 @@ main (int argc, char *argv[])
        ------------------------------------------------------------ */
 
     if ( grid.zscp.Iam == 0 ) { // process layer 0
-
 	PStatPrint (&options, &stat, &(grid.grid2d)); /* Print 2D statistics.*/
-
-        zDestroy_LU (n, &(grid.grid2d), &LUstruct);
-        zSolveFinalize (&options, &SOLVEstruct);
-    } else { // Process layers not equal 0
-        zDeAllocLlu_3d(n, &LUstruct, &grid);
-        zDeAllocGlu_3d(&LUstruct);
     }
-    
+    zDestroy_LU (n, &(grid.grid2d), &LUstruct);
+    zSolveFinalize (&options, &SOLVEstruct);
+
     zDestroy_A3d_gathered_on_2d(&SOLVEstruct, &grid);
 
     Destroy_CompRowLoc_Matrix_dist (&A);
@@ -471,16 +540,17 @@ main (int argc, char *argv[])
     zScalePermstructFree (&ScalePermstruct);
     zLUstructFree (&LUstruct);
     fclose(fp);
-    
+
     /* ------------------------------------------------------------
        RELEASE THE SUPERLU PROCESS GRID.
        ------------------------------------------------------------ */
 out:
-    if ( batch ) {
-	result_min[0] = stat.utime[FACT];   
-	result_min[1] = stat.utime[SOLVE];  
-	result_max[0] = stat.utime[FACT];   
-	result_max[1] = stat.utime[SOLVE];    
+#if 0 // the following makes sense only for coarse-grain parallel model 
+    if ( batchCount ) {
+	result_min[0] = stat.utime[FACT];
+	result_min[1] = stat.utime[SOLVE];
+	result_max[0] = stat.utime[FACT];
+	result_max[1] = stat.utime[SOLVE];
 	MPI_Allreduce(MPI_IN_PLACE, result_min, 2, MPI_FLOAT,MPI_MIN, MPI_COMM_WORLD);
 	MPI_Allreduce(MPI_IN_PLACE, result_max, 2, MPI_FLOAT,MPI_MAX, MPI_COMM_WORLD);
 	if (!myrank) {
@@ -491,10 +561,11 @@ out:
 	    fflush(stdout);
 	}
     }
+#endif
 
     superlu_gridexit3d (&grid);
-    if ( iam != -1 )PStatFree (&stat);
-    
+    if ( iam != -1 ) PStatFree (&stat);
+
     /* ------------------------------------------------------------
        TERMINATES THE MPI EXECUTION ENVIRONMENT.
        ------------------------------------------------------------ */
@@ -504,7 +575,7 @@ out:
     CHECK_MALLOC (iam, "Exit main()");
 #endif
 
-}
+} /* end MAIN */
 
 
 int
